# Define data-collection methods so we can easily stub them for tests
#
module Tweek

  def self.bootline
    @bootline ||= ::File.open("/proc/cmdline", &:read).strip
  end

  def self.balance_pid
    @balance_pid ||= `pgrep irqbalance`.strip.to_i
    raise "pgrep appears to be missing!" if $? == 127
    return @balance_pid
  end

  # Determine the queue processing interrupts for the network device
  # Empirically we observe that the convention for the name in /proc/interrupts varies
  # so some hacking required...
  def self.irqs netdev
    @irqs = ::File.open("/proc/interrupts", &:readlines).grep(/\b#{netdev}\b/)
    @irqs.reject! {|i| i !~ /\b#{netdev}-Tx-?Rx/} if @irqs.size > 1
    @irqs.map{|m| m.partition(':')[0].strip}
  end

  def self.ncores
    `getconf _NPROCESSORS_ONLN`.strip.to_i rescue 0
  end

  def self.clocksource param
     ::File.open("/sys/devices/system/clocksource/#{param}/current_clocksource",&:read).strip rescue $!.message
  end

  def self.sysctl name
    ::File.open("/proc/sys/#{name}", &:read).strip.gsub(/\s+/,' ') rescue $!.message
  end

  def self.set_sysctl name, value
    begin
      ::File.open("/proc/sys/#{name}", "w"){|f| f.write value}
      value
    rescue
      $!.message
    end
  end

  def self.net_dev_exist? name
    ::File.readable?("/sys/class/net/#{name}")
  end

  def self.net_driver name
    ::File.basename(::File.readlink("/sys/class/net/#{name}/device/driver"))
  end

  def self.blk_param blkdev, name
    ::File.open("/sys/block/#{blkdev}/queue/#{name}", &:read).strip rescue $!.message
  end

  # Receive Side Scaling - hardware
  def self.net_rss irq
    ::File.open("/proc/irq/#{irq}/smp_affinity",&:read).strip rescue $!.message
  end

  # Receive Packet Steering - software
  def self.net_rps netdev, ix
    ::File.open("/sys/class/net/#{netdev}/queues/rx-#{ix}/rps_cpus", &:read).strip rescue $!.message
  end

  def self.net_param netdev, name
    ::File.open("/sys/class/net/#{netdev}/#{name}", &:read).strip.gsub(/\s+/,' ') rescue $!.message
  end

  def self.mounted
    ::File.open("/proc/mounts", &:readlines).map{|m| m.split(' ')}
  end

  def self.xfsinfo mount
    info = `xfs_info #{mount} 2>/dev/null`.strip
    return $?.exitstatus > 0 ? nil : info
  end
end
